@@ -1,5 +1,5 @@
 /* searchutils.c - helper subroutines for grep's matchers.
-   Copyright 1992, 1998, 2000, 2007, 2009-2018 Free Software Foundation, Inc.
+   Copyright 1992, 1998, 2000, 2007, 2009-2020 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -75,18 +75,21 @@ kwsinit (bool mb_trans)
    back from CUR to the previous boundary, where a "boundary" is the
    start of a multibyte character or is an error-encoding byte.  The
    buffer ends at END (i.e., one past the address of the buffer's last
-   byte).  If CUR is already at a boundary, return 0.  If *MB_START is
-   greater than CUR, return the negative value CUR - *MB_START.
+   byte).  If CUR is already at a boundary, return 0.  If CUR is no
+   larger than *MB_START, return CUR - *MB_START without modifying
+   *MB_START or *MBCLEN.
 
    When returning zero, set *MB_START to CUR.  When returning a
-   positive value, set *MB_START to the next boundary after CUR, or to
-   END if there is no such boundary.  When returning a negative value,
-   leave *MB_START alone.  */
+   positive value, set *MB_START to the next boundary after CUR,
+   or to END if there is no such boundary, and set *MBCLEN to the
+   length of the preceding character.  */
 ptrdiff_t
-mb_goback (char const **mb_start, char const *cur, char const *end)
+mb_goback (char const **mb_start, size_t *mbclen, char const *cur,
+           char const *end)
 {
   const char *p = *mb_start;
   const char *p0 = p;
+  size_t clen;
 
   if (cur <= p)
     return cur - p;
@@ -94,13 +97,14 @@ mb_goback (char const **mb_start, char const *cur, char const *end)
   if (localeinfo.using_utf8)
     {
       p = cur;
+      clen = 1;
 
       if (cur < end && (*cur & 0xc0) == 0x80)
         for (int i = 1; i <= 3; i++)
           if ((cur[-i] & 0xc0) != 0x80)
             {
               mbstate_t mbs = { 0 };
-              size_t clen = mb_clen (cur - i, end - (cur - i), &mbs);
+              clen = mb_clen (cur - i, end - (cur - i), &mbs);
               if (i < clen && clen < (size_t) -2)
                 {
                   p0 = cur - i;
@@ -114,7 +118,7 @@ mb_goback (char const **mb_start, char const *cur, char const *end)
       mbstate_t mbs = { 0 };
       do
         {
-          size_t clen = mb_clen (p, end - p, &mbs);
+          clen = mb_clen (p, end - p, &mbs);
 
           if ((size_t) -2 <= clen)
             {
@@ -130,6 +134,8 @@ mb_goback (char const **mb_start, char const *cur, char const *end)
     }
 
   *mb_start = p;
+  if (mbclen)
+    *mbclen = clen;
   return p == cur ? 0 : cur - p0;
 }
 
@@ -192,6 +198,6 @@ wordchar_prev (char const *buf, char const *cur, char const *end)
       || (localeinfo.using_utf8 && localeinfo.sbclen[b] != -2))
     return sbwordchar[b];
   char const *p = buf;
-  cur -= mb_goback (&p, cur, end);
+  cur -= mb_goback (&p, NULL, cur, end);
   return wordchar_next (cur, end);
 }
